#!/usr/bin/php -q
<?php

    /*  Adrian SIMA, 2019
     *  dan.adrian@gmail.com
     */

    /* todo 
      polling time for each device 
     */

    error_reporting(0);

    /* get access to the database */
    include(dirname(__FILE__) . '/include/cli_check.php');

    /*  Reading ONT indexes is a cumbersome process, which can take longer than a decent
     *  amount of time expected for this operation. This is not exactly ok in large networks.
     *  At the same time, starting from the fact that a relevant variation measured values
     *  are significant at a rather large time interval, respectively several hours, I consider
     *  that reading the nodes distributed over a longer period of time is the solution 
     *  for obtaining values, sufficiently relevant and without stressing the devices.
     *
     *  From my tests I noticed that the querying a value took about 4 ms, more or less.
     *  Each ONU we have for now 5 values to read so thats mean 2 seconds for each set.
     *  Thus, at 500 ONT we will need 1000 seconds distributed in 12 hourly cycles ((2*500)/(60/5)) = ~80 sec,
     *  where 60 are $periodReadingTime and 5 minutes are crontab running interval.
     *
     *  Keep in mind! This poller must be run as separated process from Cacti poller.
     *
     */
    // minutes
    $periodReadingTime=60;


    if(!db_table_exists('ont_opl_values'))
    {
	db_execute("CREATE TABLE `ont_opl_values` (
		`id` int(11) NOT NULL,
		`index` varchar(50) NOT NULL,
		`device` varchar(150) NOT NULL,
		`time_updated` datetime NOT NULL,
		`descr` varchar(250) NOT NULL,
		`onttx` float(4,2) NOT NULL,
		`ontrx` float(4,2) NOT NULL,
		`onttvrx` float(4,2) NOT NULL,
		`oltrx` float(4,2) NOT NULL
		) ENGINE=InnoDB DEFAULT CHARSET=utf8;
		");
	db_execute("ALTER TABLE `ont_opl_values`
		ADD PRIMARY KEY (`id`),
		ADD UNIQUE KEY `id_2` (`id`),
		ADD KEY `id` (`id`);
		");
	db_execute("ALTER TABLE `ont_opl_values`
		MODIFY `id` int(11) NOT NULL AUTO_INCREMENT;
		");
    }
    if(!db_table_exists('ont_opl_devices'))
    {
	db_execute("CREATE TABLE `ont_opl_devices` (
		`device` varchar(150) NOT NULL,
		`community` varchar(150) NOT NULL
		`pollingtime` varchar(30) NOT NULL
		) ENGINE=InnoDB DEFAULT CHARSET=utf8;
		");
    }

    $query_result =  db_fetch_assoc("SELECT * 
		FROM `ont_opl_devices`;");


    for($i=0;$i<cacti_sizeof($query_result);$i++)
    {
	$mon_devices[]=array(
		    'hostname'=>$query_result[$i]['device'],
		    'community'=>$query_result[$i]['community']
		);

	/* store commumity for each device
	*/
	if(!isset($dev_com[$query_result[$i]['device']]))
	    $dev_com[$query_result[$i]['device']]=$query_result[$i]['community'];

    }


    /*  Get from device the interfaces indexes and descriptions as well. */
    $oidIndexes = array();
    for($i=0;$i<count($mon_devices);$i++)
    {
	$snmpONTInterfaces = shell_exec("snmpwalk -OnE -v 2c ".$mon_devices[$i]['hostname']." -c ".$mon_devices[$i]['community']." .1.3.6.1.4.1.2011.6.128.1.1.2.43.1.9");
	$snmpONTInterface_ar=explode("\n",$snmpONTInterfaces);

	while (list(, $value) = each($snmpONTInterface_ar))
	{
	    $snmpONT_OID_ar=explode("=",$value);
	    ereg("^.*([0-9]{10,20}.[0-9]{1,3})",$snmpONT_OID_ar[0],$oid_regex);
	    if($value)
		$oidIndexes[]=array(
		    'index'=>$oid_regex[1],
		    'hostname'=>$mon_devices[$i]['hostname'],
		    'community'=>$mon_devices[$i]['community']
	    );
	}
    }


    /*  Update database to fit all finded indexes.  */
    $query_result_ar =  db_fetch_assoc("SELECT * 
		FROM `ont_opl_values`;");



    while (list(,$device_values)= each($oidIndexes))
    {

	$insertIndex = true;
	reset($query_result_ar);

	while (list(,$dbquery_values)= each($query_result_ar)){
	
	    if (($dbquery_values['index']) === ($device_values['index']) 
		&& $dbquery_values['device'] == $device_values['hostname'])
	    {
		$insertIndex = false;
		break;
	    }
	}

	if($insertIndex == true)
	
	    db_execute("INSERT INTO `ont_opl_values` (`index`,`device`) 
		VALUES ('".$device_values['index']."','".$device_values['hostname']."');");

    }



    /*  Starting with the cactus presets, who updates values of indexes at every 5 minutes,
     *  which means 12 updates per hour (60/5). Moreover, we speed up the process by 20%
     *  so as to cover the potential of the new registered ONTs.
     */


    $query_result = db_fetch_assoc("SELECT * 
		FROM `ont_opl_values`;");
    $updateLimits = cacti_sizeof($query_result)/($periodReadingTime/5)*1.20;


    // turning minutes into secounds
    $periodReadingTime *= 60;

    $query_result =  db_fetch_assoc("SELECT * 
		FROM  `ont_opl_values`
		WHERE `time_updated` <= '".date("Y-m-d H:i:s",(time()-$periodReadingTime))."'
		LIMIT 0,".(int)$updateLimits.";");


    for($i=0;$i<cacti_sizeof($query_result);$i++)
    {
        // ONT transmit power: in dBm (value = 100)
	$onttx   = shell_exec("snmpwalk -v 2c -Oqv ".$query_result[$i]['device']." -c ".$dev_com[$query_result[$i]['device']]." .1.3.6.1.4.1.2011.6.128.1.1.2.51.1.3.".$query_result[$i]['index']);
        // ONT receive power: in dBm (value = 100)
	$ontrx   = shell_exec("snmpwalk -v 2c -Oqv ".$query_result[$i]['device']." -c ".$dev_com[$query_result[$i]['device']]." .1.3.6.1.4.1.2011.6.128.1.1.2.51.1.4.".$query_result[$i]['index']);
        // ONT CATV receive power: in dBm (value = 100)
	$onttvrx = shell_exec("snmpwalk -v 2c -Oqv ".$query_result[$i]['device']." -c ".$dev_com[$query_result[$i]['device']]." .1.3.6.1.4.1.2011.6.128.1.1.2.51.1.7.".$query_result[$i]['index']);
        // OLT receive power: in dBm (value -= 10000)
	$oltrx   = shell_exec("snmpwalk -v 2c -Oqv ".$query_result[$i]['device']." -c ".$dev_com[$query_result[$i]['device']]." .1.3.6.1.4.1.2011.6.128.1.1.2.51.1.6.".$query_result[$i]['index']);
        // ONT Description
	$descr   = shell_exec("snmpwalk -v 2c -Oqv ".$query_result[$i]['device']." -c ".$dev_com[$query_result[$i]['device']]." .1.3.6.1.4.1.2011.6.128.1.1.2.43.1.9.".$query_result[$i]['index']);

	db_execute("UPDATE `ont_opl_values`
                    SET `time_updated`='".date("Y-m-d H:i:s")."',
                        `descr`   = '".str_replace('"', '',trim($descr))."',
                        `onttx`   = '".(($onttx)   / 100)."',
                        `ontrx`   = '".(($ontrx)   / 100)."',
                        `onttvrx` = '".(($onttvrx) / 100)."',
                        `oltrx`   = '".(($oltrx    - 10000) / 100)."'
                    WHERE
                        `index` = '".$query_result[$i]['index']."';");
    }

?>